//  Demo 21: ThreadPracticeE.java
//
//   Another (More Complex) Java Application
//
//   This continues the single animation Thread 
//   approach to animating groups of objects.
//   
//   Uses the ListAnimationThread class.
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999

// Very similar to ThreadPracticeD.java, except that we have a
// linked list of MoveableSquareObjects instead of a linked list
// of MovingSquareThreads, and we have a single ListAnimation
// ThreadA object.  Also, as an added bonus, we can control the
// speed of the animation.

// Threads:
//
//   1. Main Thread -- this application.  Every running Java program
//        contains at least one Thread, the Thread main() runs on.
//   2. A ListAnimationThreadA to animate the MoveableSquares.
      
import java.awt.*;
import java.awt.event.*;
import CloseAndExit;
import MoveableSquare;
import SimpleObjectList;
import ListAnimationThreadA;

public class ThreadPracticeE extends Frame 
                             implements ActionListener {

    SimpleObjectList sobl;  // The linked list of MoveableSquare 
                            // objects.  We will add new Moveable
                            // Square objects to the list when we 
                            // create them.

    ListAnimationThreadA lata;  // the Thread that animates the
                                // linked list of MoveableSquare
                                // objects

    Image imageBuffer;    // for graphics double buffering as usual
 
    // for new MoveableSquare objects
    Label xLabel, yLabel,    // Labels for x, y components of 
          sizeLabel,         // position, size and color
          colorLabel;

    TextField xInput,        // TextFields for same
              yInput,
              sizeInput,
              colorInput;

    // for new MoveableSquare objects 
    Label rxLabel, ryLabel,  // Labels for x, y components of motion 
          dLabel;            // vector, and distance

    TextField rxInput,       // TextFields for same
              ryInput,
              dInput;

    // for animation speed
    Label pauseTimeLabel;
    TextField pauseTimeInput;

    Button launchButton;     // launch a new MoveableSquare

    int x, y, size;  // x and y components of new MoveableSquare 
                     // position, plus size of new MoveableSquare

    Color color;     // Color of new MoveableSquare

    int rx, ry, d;   // x and y components of motion vector, plus
                     // distance, in units of the motion vector
                     // length

    int pauseTime;   // pauseTime of animation, in milliseconds.
                     // The larger this is, the slower the
                     // animation runs.

    // Constructor: builds Frame and initializes variables.
    public ThreadPracticeE() {

       setLayout(new FlowLayout());

       // for new Square objects
       xLabel = new Label(" x Coord ");
       xInput = new TextField(4);
       xInput.setText("50");
       add(xLabel);  add(xInput);

       yLabel = new Label(" y Coord ");
       yInput = new TextField(4);
       yInput.setText("100");
       add(yLabel);  add(yInput);

       sizeLabel = new Label(" Size ");
       sizeInput = new TextField(4);
       sizeInput.setText("25");
       add(sizeLabel);  add(sizeInput);

       colorLabel = new Label("Color");
       colorInput = new TextField(5);
       colorInput.setText("blue");
       add(colorLabel);  add(colorInput);

       // for new MoveableSquare objects
       rxLabel = new Label(" x Motion ");
       rxInput = new TextField(3);
       rxInput.setText("1");
       add(rxLabel);  add(rxInput);

       ryLabel = new Label(" y Motion ");
       ryInput = new TextField(3);
       ryInput.setText("1");
       add(ryLabel);  add(ryInput);

       dLabel = new Label(" Distance ");
       dInput = new TextField(4);
       dInput.setText("200");
       add(dLabel);  add(dInput);

       // for animation speed
       pauseTimeLabel = new Label(" Pause Time ");
       pauseTimeInput = new TextField(4);
       pauseTimeInput.setText("20");
       add(pauseTimeLabel);   add(pauseTimeInput);

       launchButton = new Button("Launch");
       launchButton.addActionListener(this);
       add(launchButton);

       sobl = null;  // not necessary in Java, but good form!

       x = 50;  y = 100; size = 25; color = Color.blue;
       rx = 1; ry = 1;   d = 200;

       pauseTime = 20;
 
       addWindowListener(new CloseAndExit());   // as usual

    } // Constructor ThreadPracticeE()

    public void actionPerformed(ActionEvent evt) {

      MoveableSquare ms;

      SimpleObjectList first;  // new first node in linked
                               // list of objects

      String colorString;

      try {
        x = Integer.parseInt(xInput.getText());
      }
      catch (NumberFormatException e) {
        xInput.setText(Integer.toString(x)); // re-establish former
                                             // value
      }
 
      try {
        y = Integer.parseInt(yInput.getText());
      }
      catch (NumberFormatException e) {
        yInput.setText(Integer.toString(y)); // re-establish former
                                             // value
      }

      try {
        size = Integer.parseInt(sizeInput.getText());
      }
      catch (NumberFormatException e) {
        sizeInput.setText(Integer.toString(size)); 
                                      // re-establish former value
      }

      colorString = colorInput.getText();
      // always use the equals() method of the String class to
      // compare Strings for equality.  The == operator does object
      // comparison, that is it compares addresses, not content.
      if (colorString.equals("red"))
        color = Color.red;
      else if (colorString.equals("white"))
        color = Color.white;
      else if (colorString.equals("blue"))
        color = Color.blue;
      else if (colorString.equals("green"))
        color = Color.green;
      else { // restore default
        color = Color.blue;
        colorInput.setText("blue");
      }

      try {
        rx = Integer.parseInt(rxInput.getText());
      }
      catch (NumberFormatException e) {
        rxInput.setText(Integer.toString(rx)); // re-establish
                                               // former value
      }

      try {
        ry = Integer.parseInt(ryInput.getText());
      }
      catch (NumberFormatException e) {
        ryInput.setText(Integer.toString(ry)); // re-establish
                                               // former value
      }

      try {
        d = Integer.parseInt(dInput.getText());
      }
      catch (NumberFormatException e) {
        dInput.setText(Integer.toString(d)); // re-establish former
                                             // value
      }

      try {
        pauseTime = Integer.parseInt(pauseTimeInput.getText());

        // communicate new pauseTime to ListAnimationThreadA object
        lata.setPauseTime(pauseTime);
      }
      catch (NumberFormatException e) {
        pauseTimeInput.setText(Integer.toString(pauseTime));
                                             // re-establish former
                                             // value
      }

      ms = new MoveableSquare(x, y, size, 
                              rx, ry, d, color);

      // add ms to the front of the SimpleObjectList
      first = new SimpleObjectList(ms);
      if (sobl == null) sobl = first;
      else sobl = sobl.addToFront(first);

      // communicate the change to the ListAnimationThreadA
      lata.setSimpleObjectList(sobl);
      
   } // actionPerformed()

   // init(): creates the Image buffer and ListAnimationThreadA
   //         objects, and then starts the ListAnimationThreadA.
   //         Remember, we can't do this in our Constructor, because 
   //         we actually have to exist before the createImage
   //         method will work.  Also, we actually have to exist
   //         before we can be the parent of the FirstAnimation
   //         ThreadList.
   //         We have to remember to call this method from main(),
   //         or we wil get one of (1) the dreaded "pointer reference
   //         to null" Exception, or (2) the dreaded nothing happens
   //         at runtime syndrome.
   public void init() {

       imageBuffer = createImage(400, 400);

       while(imageBuffer == null) {

          try {
             Thread.sleep(50);  // 50 millisecond pause
          }
          catch(InterruptedException e) {
             // do nothing
          }

       }

       lata = new ListAnimationThreadA(sobl, this,
                                       imageBuffer, pauseTime);

       lata.start();  // start the Thread

   } // init()

   public static void main(String[ ] args) {

       ThreadPracticeE tpe = new ThreadPracticeE();
       tpe.setSize(400, 400);
       tpe.setVisible(true);
       tpe.init();  // don't forget to call this!

   } // main()

} // class ThreadPracticeE